@@ -17,21 +17,79 @@
 --   * '.:?' works as in @aeson <0.10@
 --   * '.:!' works as '.:?' in @aeson ==0.10@
 --   * Orphan instances 'FromJSON' 'Day' and 'FromJSON' 'LocalTime' for @aeson <0.10@
+--   * 'Encoding' related functionality is not added. It's present only with @aeson >=0.10@
 --
 module Data.Aeson.Compat (
-  -- * Generic decoding functions
-  decode,
-  decode',
-  decodeStrict,
-  decodeStrict',
-  AesonException(..),
-  -- * Either decoding functions
-  eitherDecode, eitherDecode', eitherDecodeStrict, eitherDecodeStrict',
-  -- * Operators
-  (.:?), (.:!),
-  -- * Re-exports
-  -- | Original 'Data.Aeson..:?' operator is not re-exported
-  module Data.Aeson,
+    -- * Encoding and decoding
+    -- ** Direct encoding
+    decode,
+    decode',
+    eitherDecode,
+    eitherDecode',
+    encode,
+    -- ** Variants for strict bytestrings
+    decodeStrict,
+    decodeStrict',
+    eitherDecodeStrict,
+    eitherDecodeStrict',
+    -- * Core JSON types
+    Value(..),
+#if MIN_VERSION_aeson(0,10,0)
+    Encoding,
+    fromEncoding,
+#endif
+    Array,
+    Object,
+    -- * Convenience types
+    DotNetTime(..),
+    -- * Type conversion
+    FromJSON(..),
+    Result(..),
+    fromJSON,
+    ToJSON(..),
+#if MIN_VERSION_aeson(0,10,0)
+    KeyValue(..),
+#else
+    (.=),
+#endif
+    -- ** Generic JSON classes and options
+    GFromJSON(..),
+    GToJSON(..),
+#if MIN_VERSION_aeson(0,11,0)
+    -- GToEncoding is introduced in 0.11.0.0
+    GToEncoding(..),
+#endif
+    genericToJSON,
+#if MIN_VERSION_aeson(0,10,0)
+    genericToEncoding,
+#endif
+    genericParseJSON,
+    defaultOptions,
+
+    -- * Inspecting @'Value's@
+    withObject,
+    withText,
+    withArray,
+    withNumber,
+    withScientific,
+    withBool,
+    -- * Constructors and accessors
+#if MIN_VERSION_aeson(0,10,0)
+    Series,
+    pairs,
+    foldable,
+#endif
+    (.:),
+    (.:?),
+    (.:!),
+    (.!=),
+    object,
+    -- * Parsing
+    json,
+    json',
+    value,
+    value',
+    Parser,
   ) where
 
 import Prelude ()
@@ -44,14 +102,15 @@ import           Data.Aeson hiding
 #endif
   )
 
-#if !MIN_VERSION_aeson (0,9,0)
 import           Data.Aeson.Parser (value, value')
+
+#if !MIN_VERSION_aeson (0,9,0)
 import qualified Data.Attoparsec.ByteString as A
 import qualified Data.Attoparsec.ByteString.Char8 as A (skipSpace)
 import qualified Data.Attoparsec.Lazy as L
 #endif
 
-import           Control.Monad.Catch
+import           Control.Monad.Catch (MonadThrow (..), Exception)
 import           Data.Aeson.Types hiding ((.:?))
 import           Data.ByteString as B
 import qualified Data.Scientific as Scientific
@@ -61,7 +120,7 @@ import           Data.Text as T
 import           Data.Typeable (Typeable)
 
 #if !MIN_VERSION_aeson(0,10,0)
-import           Data.Time (Day, LocalTime, formatTime)
+import           Data.Time (Day, LocalTime, formatTime, NominalDiffTime)
 import           Data.Time.Locale.Compat (defaultTimeLocale)
 import qualified Data.Aeson.Compat.Time as CompatTime
 #endif
@@ -221,6 +280,23 @@ instance ToJSON Day where
 
 instance ToJSON LocalTime where
   toJSON = toJSON . T.pack . formatTime defaultTimeLocale "%FT%T%Q"
+
+instance ToJSON NominalDiffTime where
+  toJSON = Number . realToFrac
+  {-# INLINE toJSON #-}
+	
+#if MIN_VERSION_aeson(0,10,0)
+  toEncoding = Encoding . E.number . realToFrac
+  {-# INLINE toEncoding #-}
+#endif
+	
+-- | /WARNING:/ Only parse lengths of time from trusted input
+-- since an attacker could easily fill up the memory of the target
+-- system by specifying a scientific number with a big exponent like
+-- @1e1000000000@.
+instance FromJSON NominalDiffTime where
+  parseJSON = withScientific "NominalDiffTime" $ pure . realToFrac
+  {-# INLINE parseJSON #-}
 #endif
 
 -----------------------------------------------------------------------
